﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using SharpDX.DirectInput;
using System.Diagnostics;

namespace JWNameSpace
{
    public class JoystickWrapper
    {
        private DirectInput directInput;
        private Dictionary<string, Joystick> acquiredSticks = new Dictionary<string, Joystick>(StringComparer.OrdinalIgnoreCase);

        public JoystickWrapper()
        {
            directInput = new DirectInput();
        }

        public int AcquireStick(string guid)
        {
            var joystick = new Joystick(directInput, new Guid(guid));

            // Set BufferSize in order to use buffered data.
            joystick.Properties.BufferSize = 128;

            joystick.Acquire();
            acquiredSticks[guid] = joystick;
            return 1;
        }

        public DeviceList GetDevices()
        {
            var deviceList = new DeviceList();

            foreach (var deviceInstance in directInput.GetDevices())
            {
                if (deviceInstance.Type != DeviceType.Joystick
                    && deviceInstance.Type != DeviceType.Gamepad
                    && deviceInstance.Type != DeviceType.FirstPerson
                    && deviceInstance.Type != DeviceType.Flight
                    && deviceInstance.Type != DeviceType.Driving)
                    continue;
                deviceList.Add(new DeviceInfo(deviceInstance));
                //Console.WriteLine(String.Format("Found stick {0}", deviceInstance.InstanceName));
            }

            return deviceList;
        }

        public DeviceReportList PollStick(string joystickGuid)
        {
            if (!acquiredSticks.ContainsKey(joystickGuid))
            {
                AcquireStick(joystickGuid);
            }
            var joystick = acquiredSticks[joystickGuid];

            joystick.Poll();
            var datas = joystick.GetBufferedData();

            var rep = new DeviceReportList() { Guid = joystickGuid };

            foreach (var state in datas)
            {
                rep.Add(new DeviceReport() { InputName = state.Offset.ToString(), Value = state.Value});
                //Debug.WriteLine(String.Format("AHK| Input {0} changed state to {1}", state.Offset.ToString(), state.Value));
            }
            return rep;
        }

        public void ConsolePollStick(string joystickGuid)
        {
            if (!acquiredSticks.ContainsKey(joystickGuid))
            {
                AcquireStick(joystickGuid);
            }
            var joystick = acquiredSticks[joystickGuid];

            while (true)
            {
                joystick.Poll();
                var datas = joystick.GetBufferedData();
                foreach (var state in datas)
                    Console.WriteLine(state);
            }
        }

    }

    public class DeviceList
    {
        public List<DeviceInfo> List { get; set; } = new List<DeviceInfo>();

        public void Add(DeviceInfo obj)
        {
            List.Add(obj);
        }

        public DeviceInfo GetIndex(int id)
        {
            return List[id];
        }

        public int Length()
        {
            return List.Count;
        }
    }

    public class DeviceInfo
    {
        public DeviceInfo(DeviceInstance deviceInstance)
        {
            Name = deviceInstance.InstanceName;
            Guid = deviceInstance.InstanceGuid.ToString();
        }

        public string Name { get; set; }
        public string Guid { get; set; }
    }

    public class DeviceReportList
    {
        public List<DeviceReport> List { get; set; } = new List<DeviceReport>();
        public string Guid { get; set; }

        public void Add(DeviceReport obj)
        {
            List.Add(obj);
        }

        public DeviceReport GetIndex(int id)
        {
            return List[id];
        }

        public int Length()
        {
            return List.Count;
        }
    }

    public class DeviceReport
    {
        public string InputName { get; set; }
        public int Value { get; set; }
    }

    public class ReturnedObject
    {
        public int BaseInt { get; set; }
    }

}
